@@ -4,7 +4,7 @@ local files     = require 'files'
 local generic   = require 'core.generic'
 local rpath     = require 'workspace.require-path'
 local vm        = require 'vm.vm'
-local collector = require 'core.collector'
+local collector = require 'core.collector' 'searcher'
 local util      = require 'utility'
 
 local TRACE        = TRACE
@@ -24,9 +24,6 @@ local sfind        = string.find
 local sformat      = string.format
 
 local getUri       = guide.getUri
-local getRoot      = guide.getRoot
-
-local ceach        = collector.each
 
 local getState     = files.getState
 
@@ -43,8 +40,6 @@ local eachBackward      = noder.eachBackward
 local eachExtend        = noder.eachExtend
 local eachSource        = noder.eachSource
 local compileAllNodes   = noder.compileAllNodes
-local compilePartNoders = noder.compilePartNodes
-local isGlobalID        = noder.isGlobalID
 local hasCall           = noder.hasCall
 
 local SPLIT_CHAR     = noder.SPLIT_CHAR
@@ -224,13 +219,13 @@ local function pushResult(status, mode, source, force)
         end
     end
 
-    local parent = source.parent
-    if parent.type == 'return' then
-        if source ~= status.source then
-            results[#results+1] = source
-            return true
-        end
-    end
+    --local parent = source.parent
+    --if parent.type == 'return' then
+    --    if source ~= status.source then
+    --        results[#results+1] = source
+    --        return true
+    --    end
+    --end
 
     return false
 end
@@ -796,7 +791,7 @@ function m.searchRefsByID(status, suri, expect, mode)
         if not requireName then
             return
         end
-        local uris = rpath.findUrisByRequirePath(requireName)
+        local uris = rpath.findUrisByRequirePath(suri, requireName)
         footprint(status, 'require:', requireName)
         for i = 1, #uris do
             local ruri = uris[i]
@@ -820,7 +815,7 @@ function m.searchRefsByID(status, suri, expect, mode)
         or mode == 'alldef'
         or field
         or hasCall(field) then
-            for _, guri in ceach('def:' .. id) do
+            for _, guri in collector:each(suri, 'def:' .. id) do
                 if uri == guri then
                     goto CONTINUE
                 end
@@ -829,14 +824,14 @@ function m.searchRefsByID(status, suri, expect, mode)
             end
         elseif mode == 'field'
         or     mode == 'allfield' then
-            for _, guri in ceach('def:' .. id) do
+            for _, guri in collector:each(suri, 'def:' .. id) do
                 if uri == guri then
                     goto CONTINUE
                 end
                 searchID(guri, id, field, uri)
                 ::CONTINUE::
             end
-            for _, guri in ceach('field:' .. id) do
+            for _, guri in collector:each(suri, 'field:' .. id) do
                 if uri == guri then
                     goto CONTINUE
                 end
@@ -844,7 +839,7 @@ function m.searchRefsByID(status, suri, expect, mode)
                 ::CONTINUE::
             end
         else
-            for _, guri in ceach(id) do
+            for _, guri in collector:each(suri, id) do
                 if crossed[guri] then
                     goto CONTINUE
                 end
@@ -872,7 +867,7 @@ function m.searchRefsByID(status, suri, expect, mode)
         or ignoredIDs[id]
         or id == 'dn:string'
         or hasCall(field) then
-            for _, guri in ceach('def:' .. id) do
+            for _, guri in collector:each(suri, 'def:' .. id) do
                 if uri == guri then
                     goto CONTINUE
                 end
@@ -880,7 +875,7 @@ function m.searchRefsByID(status, suri, expect, mode)
                 ::CONTINUE::
             end
         else
-            for _, guri in ceach(id) do
+            for _, guri in collector:each(suri, id) do
                 if crossed[guri] then
                     goto CONTINUE
                 end
