@@ -96,7 +96,7 @@ local childMap = {
     ['repeat']      = {'#', 'filter'},
     ['while']       = {'filter', '#'},
     ['in']          = {'keys', 'exps', '#'},
-    ['loop']        = {'loc', 'max', 'step', '#'},
+    ['loop']        = {'loc', 'init', 'max', 'step', '#'},
     ['if']          = {'#'},
     ['ifblock']     = {'filter', '#'},
     ['elseifblock'] = {'filter', '#'},
@@ -141,10 +141,12 @@ local childMap = {
     ['doc.type.function']  = {'#args', '#returns', 'comment'},
     ['doc.type.ltable']    = {'#fields', 'comment'},
     ['doc.type.literal']   = {'node'},
-    ['doc.type.arg']       = {'extends'},
+    ['doc.type.arg']       = {'name', 'extends'},
     ['doc.type.field']     = {'extends'},
     ['doc.overload']       = {'overload', 'comment'},
     ['doc.see']            = {'name', 'field'},
+    ['doc.version']        = {'#versions'},
+    ['doc.diagnostic']     = {'#names'},
 }
 
 ---@type table<string, fun(obj: parser.guide.object, list: parser.guide.object[])>
@@ -802,6 +804,17 @@ function m.offsetToPosition(state, offset)
     return m.offsetToPositionByLines(state.lines, offset)
 end
 
+function m.getLineRange(state, row)
+    local nextLineStart = state.lines[row + 1] or #state.lua
+    for i = nextLineStart - 1, state.lines[row], -1 do
+        local w = state.lua:sub(i, i)
+        if w ~= '\r' and w ~= '\n' then
+            return i - state.lines[row] + 1
+        end
+    end
+    return 0
+end
+
 local isSetMap = {
     ['setglobal']      = true,
     ['local']          = true,
