@@ -1,11 +1,22 @@
-local sp         = require 'bee.subprocess'
 local nonil      = require 'without-check-nil'
 local client     = require 'client'
 local platform   = require 'bee.platform'
 local completion = require 'provider.completion'
+local define     = require 'proto.define'
 
 require 'provider.semantic-tokens'
 
+local function toArray(map)
+    local array = {}
+    for k in pairs(map) do
+        array[#array+1] = k
+    end
+    table.sort(array, function (a, b)
+        return map[a] < map[b]
+    end)
+    return array
+end
+
 local m = {}
 
 local function testFileEvents(initer)
@@ -81,11 +92,11 @@ function m.getIniter()
         },
         executeCommandProvider = {
             commands = {
-                'lua.removeSpace:' .. sp:get_id(),
-                'lua.solve:'       .. sp:get_id(),
-                'lua.jsonToLua:'   .. sp:get_id(),
-                'lua.setConfig:'   .. sp:get_id(),
-                'lua.autoRequire:' .. sp:get_id(),
+                'lua.removeSpace',
+                'lua.solve',
+                'lua.jsonToLua',
+                'lua.setConfig',
+                'lua.autoRequire',
             },
         },
         foldingRangeProvider = true,
@@ -93,6 +104,14 @@ function m.getIniter()
             firstTriggerCharacter = '\n',
             moreTriggerCharacter  = nil, -- string[]
         },
+        semanticTokensProvider = {
+            legend = {
+                tokenTypes     = toArray(define.TokenTypes),
+                tokenModifiers = toArray(define.TokenModifiers),
+            },
+            range = true,
+            full  = false,
+        },
         --documentOnTypeFormattingProvider = {
         --    firstTriggerCharacter = '}',
         --},
