@@ -94,8 +94,10 @@ function m.dump(tbl, option)
     end
     local lines = {}
     local mark = {}
+    local stack = {}
     lines[#lines+1] = '{'
-    local function unpack(tbl, deep)
+    local function unpack(tbl)
+        local deep = #stack
         mark[tbl] = (mark[tbl] or 0) + 1
         local keys = {}
         local keymap = {}
@@ -153,8 +155,9 @@ function m.dump(tbl, option)
             end
             local value = tbl[key]
             local tp = type(value)
-            if option['format'] and option['format'][key] then
-                lines[#lines+1] = ('%s%s%s,'):format(TAB[deep+1], keyWord, option['format'][key](value, unpack, deep+1))
+            local format = option['format'] and option['format'][key]
+            if format then
+                lines[#lines+1] = ('%s%s%s,'):format(TAB[deep+1], keyWord, format(value, unpack, deep+1, stack))
             elseif tp == 'table' then
                 if mark[value] and mark[value] > 0 then
                     lines[#lines+1] = ('%s%s%s,'):format(TAB[deep+1], keyWord, option['loop'] or '"<Loop>"')
@@ -162,7 +165,9 @@ function m.dump(tbl, option)
                     lines[#lines+1] = ('%s%s%s,'):format(TAB[deep+1], keyWord, '"<Deep>"')
                 else
                     lines[#lines+1] = ('%s%s{'):format(TAB[deep+1], keyWord)
-                    unpack(value, deep+1)
+                    stack[#stack+1] = key
+                    unpack(value)
+                    stack[#stack] = nil
                     lines[#lines+1] = ('%s},'):format(TAB[deep+1])
                 end
             elseif tp == 'string' then
@@ -176,7 +181,7 @@ function m.dump(tbl, option)
         end
         mark[tbl] = mark[tbl] - 1
     end
-    unpack(tbl, 0)
+    unpack(tbl)
     lines[#lines+1] = '}'
     return tableConcat(lines, '\r\n')
 end
@@ -275,17 +280,23 @@ end
 
 --- 读取文件
 ---@param path string
-function m.loadFile(path)
+function m.loadFile(path, keepBom)
     local f, e = ioOpen(path, 'rb')
     if not f then
         return nil, e
     end
-    if f:read(3) ~= '\xEF\xBB\xBF' then
-        f:seek("set")
-    end
-    local buf = f:read 'a'
+    local text = f:read 'a'
     f:close()
-    return buf
+    if not keepBom then
+        if text:sub(1, 3) == '\xEF\xBB\xBF' then
+            return text:sub(4)
+        end
+        if text:sub(1, 2) == '\xFF\xFE'
+        or text:sub(1, 2) == '\xFE\xFF' then
+            return text:sub(3)
+        end
+    end
+    return text
 end
 
 --- 写入文件
@@ -638,6 +649,9 @@ function m.trim(str, mode)
 end
 
 function m.expandPath(path)
+    if type(path) ~= 'string' then
+        return nil
+    end
     if path:sub(1, 1) == '~' then
         local home = getenv('HOME')
         if not home then -- has to be Windows
@@ -685,6 +699,7 @@ function m.switch()
     return obj
 end
 
+---@param f async fun()
 function m.getUpvalue(f, name)
     for i = 1, 999 do
         local uname, value = getupvalue(f, i)
@@ -698,4 +713,12 @@ function m.getUpvalue(f, name)
     return nil, false
 end
 
+function m.stringStartWith(str, head)
+    return str:sub(1, #head) == head
+end
+
+function m.stringEndWith(str, tail)
+    return str:sub(-#tail) == tail
+end
+
 return m
